// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0.If a copy of the MPL was not distributed with this
// file, You can obtain one at https ://mozilla.org/MPL/2.0/.

#include "Angle.h"
#include <math.h>
#include "FloatSingle.h"

const float Angle::Rad2Deg = 57.29578F;
const float Angle::Deg2Rad = 0.0174532924F;

float Angle::Normalize(float angle) {
  if (!isfinite(angle))
    return angle;

  while (angle <= -180)
    angle += 360;
  while (angle > 180)
    angle -= 360;
  return angle;
}

float Angle::Clamp(float angle, float min, float max) {
  float normalizedAngle = Normalize(angle);
  float r = Float::Clamp(normalizedAngle, min, max);
  return r;
}

float Angle::Difference(float a, float b) {
  float r = Normalize(b - a);
  return r;
}

float Angle::MoveTowards(float fromAngle, float toAngle, float maxAngle) {
  float d = toAngle - fromAngle;
  float sign = signbit(d) ? -1 : 1;
  d = sign * Float::Clamp(fabs(d), 0, maxAngle);
  return fromAngle + d;
}
